/****************************************************************************
*
* FILENAME
*   TEST.CPP
*
* FUNCTION NAME
*   MAIN
*
* PURPOSE
*   Demonstrate the use of the class Modem.
*
* DESCRIPTION
*   This is a simple "dumb" terminal program. It is setup to use COM2, so it
*   might be necessary to change this. It also has a single phone number
*   coded in that also may need to be changed.
*
*   A log file of the session is created with the name DATA.TXT in the
*   current directory. This is a binary file.
*
*   The following key sequences can be used to issue commands:
*     CTRL-A  Arrest output (for flow control testing)
*     CTRL-H  Hangup the modem
*     CTRL-I  Initialize the modem
*     CTRL-Z  Exit the program
*
*   Note that all keypresses are read through a DOS call which does some
*   interpretation. For example: CTRL-S (XOFF) and CTRL-Q (XON) are NEVER
*   returned. To read all keystrokes, you must go to the bios level.
*
* EXTERNAL REFERENCES
*   class Modem
*   unsigned _stklen
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*     MODEM.OBJ
*   SSTREAM.OBJ
*    SERIAL.OBJ
*    SQUEUE.OBJ
*      UART.OBJ
*
* HISTORY
*   June 19, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* HEADER FILES
*
*****************************************************************************/

#include "modem.h"    // Definition of class Modem
#include "ascii.h"    // ASCII constants
#include <conio.h>    // Console I/O
#include <fstream.h>  // File streams

/****************************************************************************
*
* GLOBALS
*
*****************************************************************************/

// Array access to COM ports
Modem com[Modem::COMPORTS] =
{
  Modem( Modem::COM1 ),
  Modem( Modem::COM2 ),
  Modem( Modem::COM3 ),
  Modem( Modem::COM4 )
};

// Or use COM ports by name (like you would for cin, cout, cerr, clog
// with the exception that Modem streams are of class iostream type)
Modem &com1 = com[ Modem::COM1 ];
Modem &com2 = com[ Modem::COM2 ];
Modem &com3 = com[ Modem::COM3 ];
Modem &com4 = com[ Modem::COM4 ];

/****************************************************************************
*
* CONSTANTS
*
*****************************************************************************/

extern unsigned _stklen = 8192U; // A very large stack

#define PHONE_NUMBER "555-5555"

/****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void    // Nothing is returned

main()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Choose a COM port
  Modem &modem = com2;

  // enable COM port with a 4k byte receive queue, 256 byte transmit queue
  if( !modem.Enable( 4096U, 256U ) )
  {
    cout << "Cannot enable COM port!\n";
    return;
  }

  char key = 0, data;
  Boolean success;
  int slow = 0;

  // Open a log file
  ofstream datafile( "data.txt", ios::out | ios::binary );
  if(!datafile) cout << "datafile failure: open\n";

  // Initialize the modem serial port (38400bps, 8-N-1)
  modem.SetBitRate( 38400UL );
  modem.SetDataBits( SerialStream::DATA_8 );
  modem.SetStopBits( SerialStream::STOP_1 );
  modem.SetParityBit( SerialStream::PARITY_NONE );
  modem.SetFlowType( SerialStream::FLOW_HARD );

  // Initialize the modem
  modem.SetCmdPrefix( "AT" );
  modem.SetCmdSuffix( "\r" );
  modem.SetInitCmd( "Z" );
  modem.SetDialCmd( "DT" );
  modem.SetHangupCmd( "H" );
  modem.SetEscapeChar( '+' );
         
  // Instructions
  clrscr();
  cout << "Super Stupid Dumb Terminal - For test purposes only\n"
          "---------------------------------------------------\n"
          "CTRL-A = Arrest output (tests flow control)\n"
          "CTRL-D = Dial PRESET number\n"
          "CTRL-H = Hangup modem\n"
          "CTRL-I = Initialize modem\n"
          "CTRL-Z = Exit to DOS\n\n";

  // Loop until exit = control-Z pressed
  while( key != ctrl( 'Z' ) )
  {
    // Check for a keypress
    if( kbhit() )
    {
      // Check which key was pressed
      switch( key = getch() )
      {
        // Key sequence CTRL-I = initialize modem
        case ctrl( 'I' ):
          success = modem.Init();
          cout << "Modem initialization "
               << ( success ? "succeeded!\n" : "failed!\n" );
          break;

        // Key sequence CTRL-D = dial a number
        case ctrl( 'D' ):
          success = modem.Dial( PHONE_NUMBER );
          cout << "Dialing "
               << ( success ? "succeeded!\n" : "failed!\n" );
          cout << "Connect rate: " << modem.ConnectRate() << '\n';
          cout << "Carrier rate: " << modem.CarrierRate() << '\n';
          break;

        // Key sequence CTRL-H = hangup the modem
        case ctrl( 'H' ):
          success = modem.Hangup();
          cout << "Modem hangup "
               << ( success ? "succeeded!\n" : "failed!\n" );
          break;

        // Key sequence CTRL-A = arrest output
        case ctrl( 'A' ):
          slow = ~slow;
          break;

        // Default for all other keypresses
        default:
          // Check if the transmit queue is not yet full
          if( !modem.TxIsFull() )
          {
            // Output the keypress to the modem
            modem << (char) ( key & 0xff );
          }
          else
          {
            // Output an error message
            cout << "\nTx is FULL! Keypress lost.\n";
          }
          break;
      }
    }

    // If there is data waiting in the receiver then output it
    if( !slow && modem.RxIsEmpty() == False )
    {
      // Input one character from modem
      modem >> data;

      // Output data to screen
      cout << data;

      // Output same data to log file
      if( datafile ) datafile << data;
    }
  }

  // Close the log file
  if( datafile ) datafile.close();
}

/****************************************************************************/
